#include "FontUPD.h"
#include "F_SPI.h"
#include "TFT_SPI.h"
#include "diskio.h"
#include "ff.h"
#include <stdlib.h>
#include <string.h>

#define FONTSECSIZE	 	673
#define FONTINFOADDR 	1024*1024*4

_Font_Flash _FtInfo;

uint8_t *const YaHei32_Font = "/SYSTEM/FONT/YaHei3218.DZK";
uint8_t *const YaHei24_Font = "/SYSTEM/FONT/YaHei2413.DZK";
uint8_t *const YaHei20_Font = "/SYSTEM/FONT/YaHei2011.DZK";
uint8_t *const YaHei16_Font = "/SYSTEM/FONT/YaHei1609.DZK";
uint8_t *const UNIGBK_Data = "/SYSTEM/FONT/UNIGBK.BIN";
//uint8_t *const UNIGBK_Data = "/UNIGBK.BIN";

uint32_t FUPD_Progress(uint16_t x, uint16_t y, uint8_t size, uint32_t fsize, uint32_t pos)
{
    float prog;
    uint8_t t = 0XFF;
    prog = (float)pos / fsize;
    prog *= 100;
    if(t != prog)
    {
        LCD_ShowString(x + 3 * size / 2, y, 240, 320, size, "%", 0x03F0, 0xFFFF);
        t = prog;
        if(t > 100)t = 100;
        LCD_ShowNum(x, y, t, 3, size, 0x03F0, 0xFFFF);
    }
    return 0;
}

uint8_t Updata_FontX(uint16_t x, uint16_t y, uint8_t size, uint8_t *fxpath, uint8_t fx)
{
    uint32_t flashaddr = 0;
    FIL *fftemp;
    uint8_t *tempbuf;
    uint8_t res;
    uint16_t bread;
    uint32_t offx = 0;
    uint8_t rval = 0;
    fftemp = (FIL *)malloc(sizeof(FIL));
    if(fftemp == NULL)rval = 1;
    tempbuf = malloc(4096);
    if(tempbuf == NULL)rval = 1;
    res = f_open(fftemp, (const TCHAR *)fxpath, FA_READ);
    if(res)rval = 2;
    if(rval == 0)
    {
        switch(fx)
        {
        case 0:
            _FtInfo.UGBK_A = FONTINFOADDR + sizeof(_FtInfo);
            _FtInfo.UGBK_L = fftemp->fsize;
            flashaddr = _FtInfo.UGBK_A;
            break;
        case 1:
            _FtInfo.YaHei1609_A = _FtInfo.UGBK_A + _FtInfo.UGBK_L;
            _FtInfo.YaHei1609_L = fftemp->fsize;
            flashaddr = _FtInfo.YaHei1609_A;
            break;
        case 2:
            _FtInfo.YaHei2011_A = _FtInfo.YaHei1609_A + _FtInfo.YaHei1609_L;
            _FtInfo.YaHei2011_L = fftemp->fsize;
            flashaddr = _FtInfo.YaHei2011_A;
            break;
        case 3:
            _FtInfo.YaHei2413_A = _FtInfo.YaHei2011_A + _FtInfo.YaHei2011_L;
            _FtInfo.YaHei2413_L = fftemp->fsize;
            flashaddr = _FtInfo.YaHei2413_A;
            break;
        case 4:
            _FtInfo.YaHei3218_A = _FtInfo.YaHei2413_A + _FtInfo.YaHei2413_L;
            _FtInfo.YaHei3218_L = fftemp->fsize;
            flashaddr = _FtInfo.YaHei3218_A;
            break;
        }

        while(res == FR_OK)
        {
            res = f_read(fftemp, tempbuf, 4096, (UINT *)&bread);
            if(res != FR_OK)break;
            W25QXX_Write(tempbuf, offx + flashaddr, 4096);
            offx += bread;
            FUPD_Progress(x, y, size, fftemp->fsize, offx);
            if(bread != 4096)break;
        }
        f_close(fftemp);
    }
    free(fftemp);
    free(tempbuf);
    return res;
}


uint8_t update_font(uint16_t x, uint16_t y, uint8_t size, uint8_t *src)
{
    uint8_t *pname;
    uint32_t *buf;

    uint8_t res = 0;
    uint16_t i, j;
    FIL *fftemp;

    uint8_t rval = 0;
    res = 0XFF;
    _FtInfo.FontSign = 0XFF;

    f_mount(&sdfs, "0:", 1);

    pname = malloc(100);
    buf = malloc(4096);
    fftemp = (FIL *)malloc(sizeof(FIL));
    if(buf == NULL || pname == NULL || fftemp == NULL)
    {
        free(fftemp);
        free(pname);
        free(buf);
        return 1;
    }

    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)UNIGBK_Data);

    res = f_open(fftemp, (const TCHAR *)pname, FA_READ	);
    //res = f_open(fftemp,  "UNIGBK.BIN", FA_READ	);
    //res = f_open(&fsrc, "UNIGBK.BIN", FA_OPEN_EXISTING | FA_READ);

    if(res)


    {

        free(pname);
        free(buf);
        return 2;


    }

    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei16_Font);
    res = f_open(fftemp, (const TCHAR *)pname, FA_READ);
    if(res)


    {

        free(pname);
        free(buf);
        return 2;


    }
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei20_Font);
    res = f_open(fftemp, (const TCHAR *)pname, FA_READ);
    if(res)


    {

        free(pname);
        free(buf);
        return 2;


    }
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei24_Font);
    res = f_open(fftemp, (const TCHAR *)pname, FA_READ);
    if(res)


    {

        free(pname);
        free(buf);

        return 2;


    }
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei32_Font);
    res = f_open(fftemp, (const TCHAR *)pname, FA_READ);
    if(res)


    {

        free(pname);
        free(buf);
        return 2;


    }
    free(fftemp);

    LCD_ShowString(x, y, 240, 320, size, "Erasing sectors... ", 0xAA00, 0xFFFF);
    for(i = 0; i < FONTSECSIZE; i++)
    {
        FUPD_Progress(x + 20 * size / 2, y, size, FONTSECSIZE, i);
        W25QXX_Read((uint8_t *)buf, ((FONTINFOADDR / 4096) + i) * 4096, 4096);
        for(j = 0; j < 1024; j++)
        {
            if(buf[j] != 0XFFFFFFFF)break;
        }
        if(j != 1024)W25QXX_Erase_Sector((FONTINFOADDR / 4096) + i);
    }
    free(buf);
    LCD_ShowString(x, y, 240, 320, size, "Updating UNIGBK.BIN", 0xAA00, 0xFFFF);
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)UNIGBK_Data);
    res = Updata_FontX(x + 20 * size / 2, y, size, pname, 0);
    if(res)
    {
        free(pname);
        return 1;
    }
    LCD_ShowString(x, y, 240, 320, size, "Updating GBK12.BIN", 0xAA00, 0xFFFF);
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei16_Font);
    res = Updata_FontX(x + 20 * size / 2, y, size, pname, 1);
    if(res)
    {
        free(pname);
        return 2;
    }
    LCD_ShowString(x, y, 240, 320, size, "Updating GBK16.BIN", 0xAA00, 0xFFFF);
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei20_Font);
    res = Updata_FontX(x + 20 * size / 2, y, size, pname, 2);
    if(res)
    {
        free(pname);
        return 3;
    }
    LCD_ShowString(x, y, 240, 320, size, "Updating GBK24.BIN", 0xAA00, 0xFFFF);
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei24_Font);
    res = Updata_FontX(x + 20 * size / 2, y, size, pname, 3);
    if(res)
    {
        free(pname);
        return 4;
    }
    LCD_ShowString(x, y, 240, 320, size, "Updating GBK32.BIN", 0xAA00, 0xFFFF);
    strcpy((char *)pname, (char *)src);
    strcat((char *)pname, (char *)YaHei32_Font);
    res = Updata_FontX(x + 20 * size / 2, y, size, pname, 4);
    if(res)
    {
        free(pname);
        return 4;
    }

    _FtInfo.FontSign = 0XAA;
    W25QXX_Write((uint8_t *)&_FtInfo, FONTINFOADDR, sizeof(_FtInfo));

    free(pname);
    free(buf);
    return rval;
}

uint8_t Font_Init(void)
{
    uint8_t t = 0;
    W25QXX_Init();
    while(t < 10)
    {
        t++;
        W25QXX_Read((uint8_t *)&_FtInfo, FONTINFOADDR, sizeof(_FtInfo));
        if(_FtInfo.FontSign == 0XAA)break;
    }
    if(_FtInfo.FontSign != 0XAA)return 1;
    return 0;
}
