#include "TFT_SPI.h"
#include "TFT_Font.h"

static uint32_t LCD_Pow(uint8_t m, uint8_t n);

void LCD_IO_Init(void)
{
    GPIO_InitTypeDef  GPIO_InitStructure;

    GPIO_InitStructure.GPIO_Pin =  LCD_CE | LCD_DC;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIOA->BSRRL = LCD_CE;
}

//DC = 0 => CMD
//DC = 1 => DATA

void LCD_WriteReg(uint8_t Data)
{

    GPIOA->BSRRH = LCD_CE;

    GPIOA->BSRRH = LCD_DC;
    SPI1_SendByte(Data);
    GPIOA->BSRRL = LCD_CE;

}

void LCD_WriteData(uint8_t Data)
{
    GPIOA->BSRRH = LCD_CE;

    GPIOA->BSRRL = LCD_DC;
    SPI1_SendByte(Data);
    GPIOA->BSRRL = LCD_CE;

}

void LCD_Init(void)
{

    LCD_WriteReg(0xCF);
    LCD_WriteData(0x00);
    LCD_WriteData(0xD9);
    LCD_WriteData(0X30);

    LCD_WriteReg(0xED);
    LCD_WriteData(0x64);
    LCD_WriteData(0x03);
    LCD_WriteData(0X12);
    LCD_WriteData(0X81);

    LCD_WriteReg(0xE8);
    LCD_WriteData(0x85);
    LCD_WriteData(0x10);
    LCD_WriteData(0x78);

    LCD_WriteReg(0xCB);
    LCD_WriteData(0x39);
    LCD_WriteData(0x2C);
    LCD_WriteData(0x00);
    LCD_WriteData(0x34);
    LCD_WriteData(0x02);

    LCD_WriteReg(0xF7);
    LCD_WriteData(0x20);

    LCD_WriteReg(0xEA);
    LCD_WriteData(0x00);
    LCD_WriteData(0x00);

    LCD_WriteReg(0xC0);    //Power control
    LCD_WriteData(0x21);   //VRH[5:0]

    LCD_WriteReg(0xC1);    //Power control
    LCD_WriteData(0x12);   //SAP[2:0];BT[3:0]

    LCD_WriteReg(0xC5);    //VCM control
    LCD_WriteData(0x32);
    LCD_WriteData(0x3C);

    LCD_WriteReg(0xC7);    //VCM control2
    LCD_WriteData(0XC1);

    LCD_WriteReg(0x36);    // Memory Access Control
    LCD_WriteData(0x08);

    LCD_WriteReg(0x3A);
    LCD_WriteData(0x55);

    LCD_WriteReg(0xB1);
    LCD_WriteData(0x00);
    LCD_WriteData(0x18);

    LCD_WriteReg(0xB6);    // Display Function Control
    LCD_WriteData(0x0A);
    LCD_WriteData(0xA2);



    LCD_WriteReg(0xF2);    // 3Gamma Function Disable
    LCD_WriteData(0x00);

    LCD_WriteReg(0x26);    //Gamma curve selected
    LCD_WriteData(0x01);

    LCD_WriteReg(0xE0);    //Set Gamma
    LCD_WriteData(0x0F);
    LCD_WriteData(0x20);
    LCD_WriteData(0x1E);
    LCD_WriteData(0x09);
    LCD_WriteData(0x12);
    LCD_WriteData(0x0B);
    LCD_WriteData(0x50);
    LCD_WriteData(0XBA);
    LCD_WriteData(0x44);
    LCD_WriteData(0x09);
    LCD_WriteData(0x14);
    LCD_WriteData(0x05);
    LCD_WriteData(0x23);
    LCD_WriteData(0x21);
    LCD_WriteData(0x00);

    LCD_WriteReg(0XE1);    //Set Gamma
    LCD_WriteData(0x00);
    LCD_WriteData(0x19);
    LCD_WriteData(0x19);
    LCD_WriteData(0x00);
    LCD_WriteData(0x12);
    LCD_WriteData(0x07);
    LCD_WriteData(0x2D);
    LCD_WriteData(0x28);
    LCD_WriteData(0x3F);
    LCD_WriteData(0x02);
    LCD_WriteData(0x0A);
    LCD_WriteData(0x08);
    LCD_WriteData(0x25);
    LCD_WriteData(0x2D);
    LCD_WriteData(0x0F);

    LCD_WriteReg(0x11);    //Exit Sleep
    Delay(120);
    LCD_WriteReg(0x29);    //Display on
}

void LCD_Set_Addr(uint16_t xs, uint16_t ys, uint16_t xe, uint16_t ye)
{
    LCD_WriteReg(0x2a);
    LCD_WriteData(xs >> 8);
    LCD_WriteData(xs);
    LCD_WriteData(xe >> 8);
    LCD_WriteData(xe);

    LCD_WriteReg(0x2b);
    LCD_WriteData(ys >> 8);
    LCD_WriteData(ys);
    LCD_WriteData(ye >> 8);
    LCD_WriteData(ye);

    LCD_WriteReg(0x2C);

}

void LCD_Clear(uint16_t color)
{
    uint8_t VH, VL;
    uint16_t i, j;
    VH = color >> 8;
    VL = color;

    LCD_Set_Addr(0, 0, 239, 319);

    for(i = 0; i < 240; i++)
    {
        for (j = 0; j < 320; j++)
        {
            LCD_WriteData(VH);
            LCD_WriteData(VL);
        }

    }

}


//画点
//POINT_color:此点的颜色
void LCD_DrawPoint(uint16_t x, uint16_t y, uint16_t color)
{
    LCD_Set_Addr(x, y, x, y); //设置光标位置
    GPIOA->BSRRL = LCD_DC;
    LCD_WriteData(color >> 8);
    LCD_WriteData(color);
}

//在指定区域内填充指定颜色
//区域大小:
//  (xend-xsta)*(yend-ysta)
void LCD_Fill(uint16_t xsta, uint16_t ysta, uint16_t xend, uint16_t yend, uint16_t color)
{
    uint16_t i, j;
    LCD_Set_Addr(xsta, ysta, xend, yend);   //设置光标位置
    //GPIOA->BSRRL = LCD_DC;
    for(i = ysta; i <= yend; i++)
    {
        for(j = xsta; j <= xend; j++)

        {

            LCD_WriteData(color >> 8);
            LCD_WriteData(color);
        }

    }
}
//画线
//x1,y1:起点坐标
//x2,y2:终点坐标
void LCD_DrawLine(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t color)
{
    uint16_t t;
    uint16_t xerr = 0, yerr = 0, delta_x, delta_y, distance;
    int16_t incx, incy, uRow, uCol;

    delta_x = x2 - x1; //计算坐标增量
    delta_y = y2 - y1;
    uRow = x1;
    uCol = y1;
    if(delta_x > 0)incx = 1; //设置单步方向
    else if(delta_x == 0)incx = 0; //垂直线
    else
    {
        incx = -1;
        delta_x = -delta_x;
    }
    if(delta_y > 0)incy = 1;
    else if(delta_y == 0)incy = 0; //水平线
    else
    {
        incy = -1;
        delta_y = -delta_y;
    }
    if( delta_x > delta_y)distance = delta_x; //选取基本增量坐标轴
    else distance = delta_y;
    for(t = 0; t <= distance + 1; t++ ) //画线输出
    {
        LCD_DrawPoint(uRow, uCol, color); //画点
        xerr += delta_x ;
        yerr += delta_y ;
        if(xerr > distance)
        {
            xerr -= distance;
            uRow += incx;
        }
        if(yerr > distance)
        {
            yerr -= distance;
            uCol += incy;
        }
    }
}
//画矩形
void LCD_DrawRectangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t color)
{
    LCD_DrawLine(x1, y1, x2, y1, color);
    LCD_DrawLine(x1, y1, x1, y2, color);
    LCD_DrawLine(x1, y2, x2, y2, color);
    LCD_DrawLine(x2, y1, x2, y2, color);
}
//在指定位置画一个指定大小的圆
//(x,y):中心点
//r    :半径
void LCD_Draw_Circle(uint16_t x0, uint16_t y0, uint8_t r, uint16_t color)
{
    int16_t a, b;
    int16_t di;
    a = 0;
    b = r;
    di = 3 - (r << 1);       //判断下个点位置的标志
    while(a <= b)
    {
        LCD_DrawPoint(x0 - b, y0 - a, color);       //3
        LCD_DrawPoint(x0 + b, y0 - a, color);       //0
        LCD_DrawPoint(x0 - a, y0 + b, color);       //1
        LCD_DrawPoint(x0 - b, y0 - a, color);       //7
        LCD_DrawPoint(x0 - a, y0 - b, color);       //2
        LCD_DrawPoint(x0 + b, y0 + a, color);       //4
        LCD_DrawPoint(x0 + a, y0 - b, color);       //5
        LCD_DrawPoint(x0 + a, y0 + b, color);       //6
        LCD_DrawPoint(x0 - b, y0 + a, color);
        a++;
        //使用Bresenham算法画圆
        if(di < 0)di += 4 * a + 6;
        else
        {
            di += 10 + 4 * (a - b);
            b--;
        }
        LCD_DrawPoint(x0 + a, y0 + b, color);
    }
}

void LCD_ShowChar(uint16_t x, uint16_t y, uint8_t num, uint8_t size, uint8_t mode, uint16_t point, uint16_t back)
{
    uint8_t temp, t1, t;
    uint16_t y0 = y;
    uint8_t csize = (size / 8 + ((size % 8) ? 1 : 0)) * (size / 2);		//得到字体一个字符对应点阵集所占的字节数
    num = num - ' '; //得到偏移后的值（ASCII字库是从空格开始取模，所以-' '就是对应字符的字库）
    for(t = 0; t < csize; t++)
    {
        if(size == 12)temp = asc2_1206[num][t]; 	 	//调用1206字体
        else if(size == 16)temp = asc2_1608[num][t];	//调用1608字体
        else if(size == 24)temp = asc2_2412[num][t];	//调用2412字体
        else return;								//没有的字库
        for(t1 = 0; t1 < 8; t1++)
        {
            if(temp & 0x80)LCD_DrawPoint(x, y, point);
            else if(mode == 0)LCD_DrawPoint(x, y, back);
            temp <<= 1;
            y++;
            if(y >= 240)return;		//超区域了
            if((y - y0) == size) //本X刷完,再刷Y
            {
                y = y0;
                x++;
                if(x >= 320)return;	//超区域了
                break;
            }
        }
    }
}

static uint32_t LCD_Pow(uint8_t m, uint8_t n)
{
    u32 result = 1;
    while(n--)result *= m;
    return result;
}
//显示数字,高位为0,则不显示
//x,y :起点坐标
//len :数字的位数
//size:字体大小
//color:颜色
//num:数值(0~4294967295);
void LCD_ShowNum(uint16_t x, uint16_t y, uint32_t num, uint8_t len, uint8_t size, uint16_t point, uint16_t back)
{
    uint8_t t, temp;
    uint8_t enshow = 0;
    for(t = 0; t < len; t++)
    {
        temp = (num / LCD_Pow(10, len - t - 1)) % 10;
        if(enshow == 0 && t < (len - 1))
        {
            if(temp == 0)
            {
                LCD_ShowChar(x + (size / 2)*t, y, ' ', size, 0, point, back);
                continue;
            }
            else enshow = 1;

        }
        LCD_ShowChar(x + (size / 2)*t, y, temp + '0', size, 0, point, back);
    }
}
//显示字符串
//x,y:起点坐标
//width,height:区域大小
//size:字体大小
//*p:字符串起始地址
void LCD_ShowString(uint16_t x, uint16_t y, uint16_t width, uint16_t height, uint8_t size, uint8_t *p, uint16_t point, uint16_t back)
{
    uint8_t x0 = x;
    width += x;
    height += y;
    while((*p <= '~') && (*p >= ' ')) //判断是不是非法字符!
    {
        if(x >= width)
        {
            x = x0;
            y += size;
        }
        if(y >= height)break; //退出
        LCD_ShowChar(x, y, *p, size, 0, point, back);
        x += size / 2;
        p++;
    }
}

